//------------------------------------------------------------------------------
// File audio.c
//------------------------------------------------------------------------------
// Copyright (c) 2010, Texas Instruments Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Texas Instruments Inc. nor the names of its
//       contributors may be used to endorse or promote products derived from
//       this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

#include <stdio.h>
#include "tistdtypes.h"

#include "app.h"
#include "edma3.h"

//-----------------------------------------------------------------------------
// Configuration Macros
//-----------------------------------------------------------------------------

#define SAMPLES_PER_BUF     128
#define BYTES_PER_SAMPLE    4
#define N_CHANNELS          1

#define BUF_LENGTH          SAMPLES_PER_BUF * N_CHANNELS
#define BUF_SIZE            SAMPLES_PER_BUF * N_CHANNELS * BYTES_PER_SAMPLE

//-----------------------------------------------------------------------------
// Static Variable Declarations
//-----------------------------------------------------------------------------

static Uint32 rcv_ping_L[BUF_LENGTH], rcv_pong_L[BUF_LENGTH],
              rcv_ping_R[BUF_LENGTH], rcv_pong_R[BUF_LENGTH],
              xmt_ping_L[BUF_LENGTH], xmt_pong_L[BUF_LENGTH],
              xmt_ping_R[BUF_LENGTH], xmt_pong_R[BUF_LENGTH];

//-----------------------------------------------------------------------------
// Functions
//-----------------------------------------------------------------------------

static void process(void *in_buf, void *out_buf)
{
    // process data from in_buf and but result in out_buf

    memcpy(out_buf, in_buf, BUF_SIZE); // (placeholder)
}

static void audio_run(void)
{
    while (1)
    {
        // ping
        SEM_pend(&rcv_ping_sem, SYS_FOREVER);
        SEM_pend(&xmt_ping_sem, SYS_FOREVER);
        process(rcv_ping_L, xmt_ping_L);
        process(rcv_ping_R, xmt_ping_R);

        // pong
        SEM_pend(&rcv_pong_sem, SYS_FOREVER);
        SEM_pend(&xmt_pong_sem, SYS_FOREVER);
        process(rcv_pong_L, xmt_pong_L);
        process(rcv_pong_R, xmt_pong_R);
    }
}

//-----------------------------------------------------------------------------
// Static TSK Function
//-----------------------------------------------------------------------------

int audio_tsk_fxn()
{
    LOG_printf(&trace, "[audio TSK]: starting audio TSK...");

    // configure and start EDMA channels
    LOG_printf(&trace, "[audio TSK]: setting up EDMA...");

    memset(rcv_ping_L, 0, BUF_SIZE);
    memset(rcv_pong_L, 0, BUF_SIZE);
    memset(rcv_ping_R, 0, BUF_SIZE);
    memset(rcv_pong_R, 0, BUF_SIZE);
    memset(xmt_ping_L, 0, BUF_SIZE);
    memset(xmt_pong_L, 0, BUF_SIZE);
    memset(xmt_ping_R, 0, BUF_SIZE);
    memset(xmt_pong_R, 0, BUF_SIZE);

    setup_edma_pingpong_rcv(mcasp_rcv_register, rcv_ping_L, rcv_pong_L,
                            rcv_ping_R, rcv_pong_R,
                            BYTES_PER_SAMPLE, SAMPLES_PER_BUF);

    setup_edma_pingpong_xmt(xmt_ping_L, xmt_pong_L,
                            xmt_ping_R, xmt_pong_R, mcasp_xmt_register,
                            BYTES_PER_SAMPLE, SAMPLES_PER_BUF);

    EdmaEnableChannel(EDMA_MCASPRXCH, 0);
    EdmaEnableChannel(EDMA_MCASPTXCH, 0);

    EdmaIntEnable(EDMA_RCV_PING_TCC);
    EdmaIntEnable(EDMA_RCV_PONG_TCC);
    EdmaIntEnable(EDMA_XMT_PING_TCC);
    EdmaIntEnable(EDMA_XMT_PONG_TCC);

    // setup codec and McASP
    LOG_printf(&trace, "[audio TSK]: initializing audio device...");
    mcasp_open();

    // run audio test
    LOG_printf(&trace, "[audio TSK]: running audio processing loop...");
    audio_run();

    // kill codec and McASP
    LOG_printf(&trace, "[audio TSK]: closing audio device...");
    mcasp_close();

    // close application
    LOG_printf(&trace, "[audio TSK]: audio TSK exited!");

    return 0;
}

//------------------------------------------------------------------------------
// End of File audio.c
//------------------------------------------------------------------------------
