//------------------------------------------------------------------------------
// File edma3.c
//------------------------------------------------------------------------------
// Copyright (c) 2010, Texas Instruments Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Texas Instruments Inc. nor the names of its
//       contributors may be used to endorse or promote products derived from
//       this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

#include <stdio.h>

#include "edma3.h"
#include "app.h"

//------------------------------------------------------------------------------
// Local Variables
//------------------------------------------------------------------------------

static CSL_Edma3ccRegsOvly edmaCcRegs = (CSL_Edma3ccRegsOvly)CSL_EDMA3CC_0_REGS;

//------------------------------------------------------------------------------
// Functions
//------------------------------------------------------------------------------

void edmaInit()
{
    edmaCcRegs->ECR  = 0xffffffff; // clear events  0 -> 31
    edmaCcRegs->SECR = 0xffffffff; // clear secondary events  0 -> 31
    edmaCcRegs->IECR = 0xffffffff; // disable all interrupts
    edmaCcRegs->ICR  = 0xffffffff; // clear all pending interrupts
}

void setup_edma_pingpong_xmt(void *src_ping_L, void *src_pong_L,
                             void *src_ping_R, void *src_pong_R,
                             void *dst,
                             Uint32 sample_size, Uint32 sample_count)
{
    CSL_Edma3ccParamSetRegs param;
    Uint32 ping_offset = (Uint32)src_ping_R - (Uint32)src_ping_L,
           pong_offset = (Uint32)src_pong_R - (Uint32)src_pong_L;
    Int32 ping_c_idx = -ping_offset + sample_size,
          pong_c_idx = -pong_offset + sample_size;

    // 1. setup channel PaRAM slot
    param.OPT = (EDMA_XMT_PING_TCC << 12) | (1 << 20);      // transfer complete interrupt enabled
    param.SRC = (Uint32)src_ping_L;
    param.A_B_CNT = (2 << 16) | sample_size;                // actual format: BCNT|ACNT
    param.DST = (Uint32)dst;
    param.SRC_DST_BIDX = (0 << 16) | ping_offset;           // actual format: DSTBIDX|SCRBIDX
    param.LINK_BCNTRLD = (2 << 16) | (EDMA_XMTPONG * 0x20); // actual format: BCNTRLD|LINK
    param.SRC_DST_CIDX = (0 << 16) | (ping_c_idx & 0x0000FFFF); // actual format: DSTCIDX|SRCCIDX
    param.CCNT = sample_count;
    edmaWritePaRAM(EDMA_MCASPTXCH, &param);

    // 2. setup ping PaRAM set (to be reloaded later)
    edmaWritePaRAM(EDMA_XMTPING, &param);

    // 3. setup pong PaRAM set (to be loaded and reloaded later)
    param.OPT = (EDMA_XMT_PONG_TCC << 12) | (1 << 20);      // transfer complete interrupt enabled    
    param.SRC_DST_BIDX = (0 << 16) | pong_offset;           // actual format: DSTBIDX|SCRBIDX
    param.LINK_BCNTRLD = (2 << 16) | (EDMA_XMTPING * 0x20); // actual format: BCNTRLD|LINK
    param.SRC_DST_CIDX = (0 << 16) | (pong_c_idx & 0x0000FFFF); // actual format: DSTCIDX|SRCCIDX  
    param.SRC = (Uint32)src_pong_L;
    edmaWritePaRAM(EDMA_XMTPONG, &param);
}

void setup_edma_pingpong_rcv(void *src,
                             void *dst_ping_L, void *dst_pong_L,
                             void *dst_ping_R, void *dst_pong_R,
                             Uint32 sample_size, Uint32 sample_count)
{
    CSL_Edma3ccParamSetRegs param;
    Uint32 ping_offset = (Uint32)dst_ping_R - (Uint32)dst_ping_L,
           pong_offset = (Uint32)dst_pong_R - (Uint32)dst_pong_L;
    Int16 ping_c_idx = -ping_offset + sample_size,
          pong_c_idx = -pong_offset + sample_size;

    // 1. setup channel PaRAM slot
    param.OPT = (EDMA_RCV_PING_TCC << 12) | (1 << 20);      // transfer complete interrupt enabled
    param.SRC = (Uint32)src;
    param.A_B_CNT = (2 << 16) | sample_size;                // actual format: BCNT|ACNT
    param.DST = (Uint32)dst_ping_L;
    param.SRC_DST_BIDX = (ping_offset << 16) | 0;           // actual format: DSTBIDX|SCRBIDX
    param.LINK_BCNTRLD = (2 << 16) | (EDMA_RCVPONG * 0x20); // actual format: BCNTRLD|LINK
    param.SRC_DST_CIDX = ((ping_c_idx << 16) & 0xFFFF0000) | 0; // actual format: DSTCIDX|SRCCIDX
    param.CCNT = sample_count;
    edmaWritePaRAM(EDMA_MCASPRXCH, &param);

    // 2. setup ping PaRAM set (to be reloaded later)
    edmaWritePaRAM(EDMA_RCVPING, &param);

    // 3. setup pong PaRAM set (to be loaded and reloaded later)
    param.OPT = (EDMA_RCV_PONG_TCC << 12) | (1 << 20);      // transfer complete interrupt enabled    
    param.SRC_DST_BIDX = (pong_offset << 16) | 0;           // actual format: DSTBIDX|SCRBIDX
    param.LINK_BCNTRLD = (2 << 16) | (EDMA_RCVPING * 0x20); // actual format: BCNTRLD|LINK
    param.SRC_DST_CIDX = ((pong_c_idx << 16) & 0xFFFF0000) | 0; // actual format: DSTCIDX|SRCCIDX
    param.DST = (Uint32)dst_pong_L;
    edmaWritePaRAM(EDMA_RCVPONG, &param);
}

void edmaWritePaRAM(Int ParamNum, CSL_Edma3ccParamSetRegs *ptrParamInfo)
{
    edmaCcRegs->PARAMSET[ParamNum].OPT = ptrParamInfo->OPT;
    edmaCcRegs->PARAMSET[ParamNum].SRC = ptrParamInfo->SRC;
    edmaCcRegs->PARAMSET[ParamNum].A_B_CNT = ptrParamInfo->A_B_CNT;
    edmaCcRegs->PARAMSET[ParamNum].DST = ptrParamInfo->DST;
    edmaCcRegs->PARAMSET[ParamNum].SRC_DST_BIDX = ptrParamInfo->SRC_DST_BIDX;
    edmaCcRegs->PARAMSET[ParamNum].LINK_BCNTRLD = ptrParamInfo->LINK_BCNTRLD; 
    edmaCcRegs->PARAMSET[ParamNum].SRC_DST_CIDX = ptrParamInfo->SRC_DST_CIDX;
    edmaCcRegs->PARAMSET[ParamNum].CCNT = ptrParamInfo->CCNT;
}

void edmaSetEvent(Int channelNumber)
{
    int mask;

    mask = 1 << channelNumber;
    edmaCcRegs->ESR = mask;
}

void EdmaIntClear(Int channelNumber)
{
    edmaCcRegs->ICR = 1 << channelNumber;    
}

void EdmaIntEnable(Int intNumber)
{
    edmaCcRegs->IESR = 1 << intNumber;
    edmaCcRegs->DRA[0].DRAE &= ~(1 << intNumber);
    edmaCcRegs->DRA[1].DRAE |= 1 << intNumber; 
}

// not a tested function
void EdmaIntDisable(Int intNumber)
{
    int mask;

    mask = 1 << intNumber;
    edmaCcRegs->DRA[0].DRAE &= ~mask;
}

void EdmaEnableChannel(Int channelNumber, Int QueueNumber)
{
    int mask;

    edmaCcRegs->EMCR = 1 << channelNumber;
    edmaCcRegs->SECR = 1 << channelNumber;

    QueueNumber &= 1; // only 0 or 1 are valid
   
    mask = 0x07 << (4 * (channelNumber & 0x07) );
   
    edmaCcRegs->DMAQNUM[channelNumber >> 3] &= ~mask;
    edmaCcRegs->DMAQNUM[channelNumber >> 3] |= QueueNumber << (4 * (channelNumber & 0x07) );        
   
    edmaCcRegs->EESR = 1 << channelNumber;
}

void EdmaDisableChannel(Int channelNumber)
{
    edmaCcRegs->EECR = 1 << channelNumber;
}

void edmaWaitForEvent(int event)
{
    Uint32 mask = 1 << event;

    while((edmaCcRegs->IPR & mask) == NULL)
        asm(" nop");

    EdmaIntClear(event);
}

void edma3ccIsr()
{
    while(edmaCcRegs->IPR)
    {
        if (edmaCcRegs->IPR & (1 << EDMA_RCV_PING_TCC))    // receive channel (ping)
        {
            EdmaIntClear(EDMA_RCV_PING_TCC);
            SEM_post(&rcv_ping_sem);
            //LOG_printf(&trace, "[ISR]: receive ping!");
        }

        if (edmaCcRegs->IPR & (1 << EDMA_RCV_PONG_TCC))    // receive channel (pong)
        {
            EdmaIntClear(EDMA_RCV_PONG_TCC);
            SEM_post(&rcv_pong_sem);
            //LOG_printf(&trace, "[ISR]: receive pong!");
        }

        if (edmaCcRegs->IPR & (1 << EDMA_XMT_PING_TCC))    // transmit channel (ping)
        {
            EdmaIntClear(EDMA_XMT_PING_TCC);
            SEM_post(&xmt_ping_sem);
            //LOG_printf(&trace, "[ISR]: transmit ping!");
        }

        if (edmaCcRegs->IPR & (1 << EDMA_XMT_PONG_TCC))    // transmit channel (pong)
        {
            EdmaIntClear(EDMA_XMT_PONG_TCC);
            SEM_post(&xmt_pong_sem);
            //LOG_printf(&trace, "[ISR]: transmit pong!");
        }
    }

    return;
}

//------------------------------------------------------------------------------
// End of File edma3.c
//------------------------------------------------------------------------------
